@@ -2,7 +2,7 @@
 import os
 import logging
 
-from qtpy.QtWidgets import QApplication
+from qtpy.QtWidgets import QApplication, QWidget
 
 from ..config import STYLESHEET, STYLESHEET_INCLUDE_DEFAULT
 
@@ -59,7 +59,6 @@ def apply_stylesheet(stylesheet_file_path=None, widget=None):
 
     widget.setStyleSheet(style)
 
-
 def _get_style_data(stylesheet_file_path=None):
     """
     Read the global stylesheet file and provide the style data as a str.
@@ -116,3 +115,6 @@ def _get_style_data(stylesheet_file_path=None):
                     GLOBAL_STYLESHEET,
                     str(ex)))
     return __style_data
+
+def global_style():
+    return _get_style_data()
