@@ -127,7 +127,6 @@ def refresh_style(widget):
             # Widget was probably destroyed
             logger.debug('Error while refreshing stylesheet. %s ', ex)
 
-
 class PyDMPrimitiveWidget(object):
     """
     Primitive class that determines that a given widget is a PyDMWidget.
@@ -150,6 +149,27 @@ class PyDMPrimitiveWidget(object):
             # and not at the Designer
             self.installEventFilter(self)
 
+    def __init_subclass__(cls, new_properties={}):
+        """
+        Adds or redefines rule-triggered property configuration for derivative
+        classes.
+
+        Parameters
+        ----------
+        new_properties: dict
+            A dictionary containing the properties that can be modified
+            through rule triggers. The format of this dictionary must
+            follow the one for entries in PyDMPrimitiveWidget.RULE_PROPERTIES.
+            Namely, the key should be a name to be displayed by the Rule
+            Editor (in designer), and the value a list containing two elements:
+            a string naming the method in the class that will handle the
+            rule dispatch, and a type matching the one that we expect to
+            receive from the PV value.
+        """
+        if new_properties:
+            cls.RULE_PROPERTIES = cls.RULE_PROPERTIES.copy()
+            cls.RULE_PROPERTIES.update(new_properties)
+
     @staticmethod
     def get_designer_icon():
         """Icon for usage in Qt designer."""
@@ -424,7 +444,7 @@ class TextFormatter(object):
         # from the PV
         if self._precision_from_pv is not None and self._precision_from_pv:
             return
-        if new_prec and self._prec != int(new_prec) and new_prec >= 0:
+        if new_prec and self._user_prec != int(new_prec) and new_prec >= 0:
             self._user_prec = int(new_prec)
             if not is_qt_designer() or config.DESIGNER_ONLINE:
                 self.value_changed(self.value)
@@ -560,7 +580,12 @@ class TextFormatter(object):
         self.update_format_string()
 
 
-class PyDMWidget(PyDMPrimitiveWidget):
+_positionRuleProperties = {
+    'Position - X': ['setX', int],
+    'Position - Y': ['setY', int]
+    }
+
+class PyDMWidget(PyDMPrimitiveWidget, new_properties=_positionRuleProperties):
     """
     PyDM base class for Read-Only widgets.
     This class implements all the functions of connection, alarm
@@ -582,13 +607,6 @@ class PyDMWidget(PyDMPrimitiveWidget):
     def __init__(self, init_channel=None):
         super(PyDMWidget, self).__init__()
 
-        if not all([prop in PyDMPrimitiveWidget.RULE_PROPERTIES for prop in
-                    ['Position - X', 'Position - Y']]):
-            PyDMWidget.RULE_PROPERTIES = PyDMPrimitiveWidget.RULE_PROPERTIES.copy()
-            PyDMWidget.RULE_PROPERTIES.update(
-                {'Position - X': ['setX', int],
-                 'Position - Y': ['setY', int]})
-
         self._connected = True
         self._channel = None
         self._channels = list()
