@@ -1,12 +1,14 @@
 import functools
 import json
 import warnings
+from typing import Optional
 from qtpy.QtGui import QColor, QBrush
 from qtpy.QtCore import Signal, Slot, Property, QTimer, Qt, QEvent, QRect
 from qtpy.QtWidgets import QToolTip
 from .. import utilities
 from pyqtgraph import AxisItem, PlotWidget, PlotDataItem, mkPen, ViewBox, InfiniteLine, SignalProxy
 from collections import OrderedDict
+from typing import Optional
 from .base import PyDMPrimitiveWidget, widget_destroyed
 from .multi_axis_plot import MultiAxisPlot
 
@@ -67,6 +69,7 @@ class BasePlotCurveItem(PlotDataItem):
 
     def __init__(self, color=None, lineStyle=None, lineWidth=None, yAxisName=None, **kws):
         self._color = QColor('white')
+        self._thresholdColor = QColor('white')
         self._pen = mkPen(self._color)
         if lineWidth is not None:
             self._pen.setWidth(lineWidth)
@@ -83,6 +86,13 @@ class BasePlotCurveItem(PlotDataItem):
         else:
             self._y_axis_name = yAxisName
 
+        # Bar related items will only be set if the user wants to render the plot as a bar graph
+        self.bar_width = None
+        # Value above or below these thresholds will be drawn in the threshold color on bar graphs
+        self.upper_threshold = None
+        self.lower_threshold = None
+        self.bar_graph_item = None
+
         if hasattr(self, "channels"):
             self.destroyed.connect(functools.partial(widget_destroyed,
                                                      self.channels))
@@ -145,6 +155,42 @@ class BasePlotCurveItem(PlotDataItem):
         self.setPen(self._pen)
         self.setSymbolPen(self._color)
 
+    @property
+    def threshold_color_string(self) -> str:
+        """
+        A string representation of the threshold color used for the bar graph.  This string
+        will be a hex color code, like #FF00FF, or an SVG spec color name, if
+        a name exists for the color.
+
+        Returns
+        -------
+        str
+        """
+        return str(utilities.colors.svg_color_from_hex(self.threshold_color.name(),
+                                                       hex_on_fail=True))
+
+    @property
+    def threshold_color(self) -> QColor:
+        """
+        The color used for bars exceeding either the upper or lower thresholds.
+
+        Returns
+        -------
+        QColor
+        """
+        return self._thresholdColor
+
+    @threshold_color.setter
+    def threshold_color(self, new_color: QColor):
+        """
+        Set the color used for bars exceeding either the upper or lower thresholds.
+
+        Parameters
+        -------
+        new_color: QColor
+        """
+        self._thresholdColor = new_color
+
     @property
     def y_axis_name(self):
         """
@@ -267,6 +313,29 @@ class BasePlotCurveItem(PlotDataItem):
         """
         self.setSymbolSize(int(new_size))
 
+    def setBarGraphInfo(self, bar_width: Optional[float] = 1.0, upper_threshold: Optional[float] = None,
+                        lower_threshold: Optional[float] = None, color: Optional[QColor] = None) -> None:
+        """
+        Set the attributes associated with displaying a plot as a bar graph. These will only be set
+        if the plot is to be rendered as a bar graph. And any or all of them may be omitted even if it
+        is a bar graph. Omitting color parameters will result in the plot displaying a uniform color.
+
+        Parameters
+        ----------
+        bar_width: float, optional
+            The width of all bars displayed on the plot
+        upper_threshold: float, optional
+            Any bar above this value will be drawn in the threshold color
+        lower_threshold: float, optional
+            Any bar below this value will be drawn in the threshold color.
+        color: QColor, optional
+            The color to draw bars exceeding either threshold in.
+        """
+        self.bar_width = bar_width
+        self.upper_threshold = upper_threshold
+        self.lower_threshold = lower_threshold
+        self.threshold_color = color
+
     def to_dict(self):
         """
         Returns an OrderedDict representation with values for all properties
@@ -282,7 +351,11 @@ class BasePlotCurveItem(PlotDataItem):
                             ("lineWidth", self.lineWidth),
                             ("symbol", self.symbol),
                             ("symbolSize", self.symbolSize),
-                            ("yAxisName", self.y_axis_name)])
+                            ("yAxisName", self.y_axis_name),
+                            ("barWidth", self.bar_width),
+                            ("upperThreshold", self.upper_threshold),
+                            ("lowerThreshold", self.lower_threshold),
+                            ("thresholdColor", self.threshold_color_string)])
 
     def close(self):
         pass
@@ -294,17 +367,21 @@ class BasePlotAxisItem(AxisItem):
 
     Parameters
     ----------
-    axisName: str
+    name: str
         The name of the axis
-    axisOrientation: str, optional
+    orientation: str, optional
         The orientation of this axis. The default for this value is 'left'. Must be set to either 'right', 'top',
         'bottom', or 'left'. See: https://pyqtgraph.readthedocs.io/en/latest/graphicsItems/axisitem.html
-    axisMinRange: float, optional
+    label: str, optional
+        The label to be displayed along the axis
+    minRange: float, optional
         The minimum value to be displayed on this axis
-    axisMaxRange: float, optional
+    maxRange: float, optional
         The maximum value to be displayed on this axis
-    axisAutoRange: bool, optional
+    autoRange: bool, optional
         Whether or not this axis should automatically update its range as it receives new data
+    logMode: bool, optional
+        If true, this axis will start in logarithmic mode, will be linear otherwise
     **kws: optional
         Extra arguments for CSS style options for this axis
     """
@@ -312,15 +389,17 @@ class BasePlotAxisItem(AxisItem):
     axis_orientations = OrderedDict([('Left', 'left'),
                                      ('Right', 'right')])
 
-    def __init__(self, name, orientation='left', minRange=-1.0,
-                 maxRange=1.0, autoRange=True, **kws):
+    def __init__(self, name, orientation='left', label=None, minRange=-1.0,
+                 maxRange=1.0, autoRange=True, logMode=False, **kws):
         super(BasePlotAxisItem, self).__init__(orientation, **kws)
 
         self._name = name
         self._orientation = orientation
+        self._label = label
         self._min_range = minRange
         self._max_range = maxRange
         self._auto_range = autoRange
+        self._log_mode = logMode
 
     @property
     def name(self):
@@ -367,6 +446,16 @@ class BasePlotAxisItem(AxisItem):
         """
         self._orientation = orientation
 
+    @property
+    def label_text(self) -> str:
+        """ Return the label to be displayed along this axis. """
+        return self._label
+
+    @label_text.setter
+    def label_text(self, label: str):
+        """ Set the label to be displayed along this axis """
+        self._label = label
+
     @property
     def min_range(self):
         """
@@ -433,6 +522,28 @@ class BasePlotAxisItem(AxisItem):
         """
         self._auto_range = auto_range
 
+    @property
+    def log_mode(self):
+        """
+        Return whether or not this axis is using logarithmic mode
+
+        Returns
+        -------
+        bool
+        """
+        return self._log_mode
+
+    @log_mode.setter
+    def log_mode(self, log_mode):
+        """
+        Set whether or not this axis is using logarithmic mode
+
+        Parameters
+        ----------
+        log_mode: bool
+        """
+        self._log_mode = log_mode
+
     def to_dict(self):
         """
         Returns an OrderedDict representation with values for all properties
@@ -444,9 +555,11 @@ class BasePlotAxisItem(AxisItem):
         """
         return OrderedDict([("name", self._name),
                             ("orientation", self._orientation),
+                            ("label", self._label),
                             ("minRange", self._min_range),
                             ("maxRange", self._max_range),
-                            ("autoRange", self._auto_range)])
+                            ("autoRange", self._auto_range),
+                            ("logMode", self._log_mode)])
 
 
 class BasePlot(PlotWidget, PyDMPrimitiveWidget):
@@ -456,6 +569,10 @@ class BasePlot(PlotWidget, PyDMPrimitiveWidget):
         # First create a custom MultiAxisPlot to pass to the base PlotWidget class to support multiple y axes. Note
         # that this plot will still function just fine in the case the user doesn't need additional y axes.
         plotItem = MultiAxisPlot(axisItems=axisItems)
+        if axisItems is None or 'left' not in axisItems:
+            # The pyqtgraph PlotItem.setAxisItems() will always add an an AxisItem called left whether you asked
+            # it to or not. This will clear it if not specifically requested.
+            plotItem.removeAxis('left')
         super(BasePlot, self).__init__(parent=parent, background=background, plotItem=plotItem)
 
         self.plotItem = plotItem
@@ -478,7 +595,7 @@ class BasePlot(PlotWidget, PyDMPrimitiveWidget):
         self.redraw_timer = QTimer(self)
         self.redraw_timer.timeout.connect(self.redrawPlot)
 
-        self._redraw_rate = 30  # Redraw at 30 Hz by default.
+        self._redraw_rate = 1  # Redraw at 1 Hz by default.
         self.maxRedrawRate = self._redraw_rate
         self._axes = []
         self._curves = []
@@ -524,8 +641,12 @@ class BasePlot(PlotWidget, PyDMPrimitiveWidget):
 
     def addCurve(self, plot_data_item, curve_color=None, y_axis_name=None):
         """
-        Adds a curve to this plot. If the y axis parameters are specified, either link this curve to an existing
-        axis if that axis is already part of this plot, or create a new one and link the curve to it.
+        Adds a curve to this plot. 
+
+        If the y axis parameters are specified, either link this curve to an
+        existing axis if that axis is already part of this plot, or create a
+        new one and link the curve to it.
+
         Parameters
         ----------
         plot_data_item: BasePlotCurveItem
@@ -533,8 +654,8 @@ class BasePlot(PlotWidget, PyDMPrimitiveWidget):
         curve_color: QColor, optional
             The color to draw the curve and axis label in
         y_axis_name: str, optional
-            The name of the axis to link the curve with. If this is the first time seeing this name,
-            then a new axis will be created for it.
+            The name of the axis to link the curve with. If this is the first
+            time seeing this name, then a new axis will be created for it.
         """
 
         if curve_color is None:
@@ -545,9 +666,14 @@ class BasePlot(PlotWidget, PyDMPrimitiveWidget):
         self._curves.append(plot_data_item)
 
         if y_axis_name is None:
-            # If the user did not name the axis, use the default ones. Note: multiple calls to setAxisItems() are ok
-            self.plotItem.setAxisItems()
-            self.addItem(plot_data_item)
+            if utilities.is_qt_designer():
+                # If we are just in designer, add an axis that will not conflict with the pyqtgraph default
+                self.addAxis(plot_data_item=plot_data_item, name='Axis 1', orientation='left')
+            # If not in designer and the user did not name the axis, use the pyqtgraph default one named left
+            elif 'left' not in self.plotItem.axes:
+                self.addAxis(plot_data_item=plot_data_item, name='left', orientation='left')
+            else:
+                self.plotItem.linkDataToAxis(plot_data_item, 'left')
         elif y_axis_name in self.plotItem.axes:
             # If the user has chosen an axis that already exists for this curve, simply link the data to that axis
             self.plotItem.linkDataToAxis(plot_data_item, y_axis_name)
@@ -560,9 +686,14 @@ class BasePlot(PlotWidget, PyDMPrimitiveWidget):
             if chan:
                 chan.connect()
 
-    def addAxis(self, plot_data_item, name, orientation, min_range=-1.0, max_range=1.0, enable_auto_range=True):
+    def addAxis(self, plot_data_item: BasePlotCurveItem, name: str, orientation: str,
+                label: Optional[str] = None, min_range: Optional[float] = -1.0,
+                max_range: Optional[float] = 1.0, enable_auto_range: Optional[bool] = True,
+                log_mode: Optional[bool] = False):
         """
-        Create an AxisItem with the input name and orientation, and add it to this plot.
+        Create an AxisItem with the input name and orientation, and add it to
+        this plot.
+
         Parameters
         ----------
         plot_data_item: BasePlotCurveItem
@@ -571,13 +702,17 @@ class BasePlot(PlotWidget, PyDMPrimitiveWidget):
             The name that will be assigned to this axis
         orientation: str
             The orientation of this axis, must be in 'left' or 'right'
+        label: str, optional
+            The label to be displayed along this axis
         min_range: float
             The minimum range to display on the axis
         max_range: float
             The maximum range to display on the axis
-        enable_auto_range: bool
+        enable_auto_range: bool, optional
             Whether or not to use autorange for this axis. Min and max range will not be respected
             when data goes out of range if this is set to True
+        log_mode: bool, optional
+            Whether or not this axis should start out in logarithmic mode.
 
         Raises
         ------
@@ -588,9 +723,13 @@ class BasePlot(PlotWidget, PyDMPrimitiveWidget):
         if name in self.plotItem.axes:
             return
 
-        axis = BasePlotAxisItem(name=name, orientation=orientation, minRange=min_range,
-                                maxRange=max_range, autoRange=enable_auto_range)
+        axis = BasePlotAxisItem(name=name, orientation=orientation, label=label, minRange=min_range,
+                                maxRange=max_range, autoRange=enable_auto_range, logMode=log_mode)
+        axis.setLabel(text=label)
         axis.enableAutoSIPrefix(False)
+        if plot_data_item is not None:
+            plot_data_item.setLogMode(False, log_mode)
+        axis.setLogMode(log_mode)
         self._axes.append(axis)
         # If the x axis is just timestamps, we don't want autorange on the x axis
         setXLink = hasattr(self, '_plot_by_timestamps') and self._plot_by_timestamps
@@ -692,11 +831,8 @@ class BasePlot(PlotWidget, PyDMPrimitiveWidget):
         return self.getAxis('bottom')._pen.color()
 
     def setAxisColor(self, color):
-        if self.getAxis('bottom')._pen.color() != color:
-            self.getAxis('bottom').setPen(color)
-            self.getAxis('left').setPen(color)
-            self.getAxis('top').setPen(color)
-            self.getAxis('right').setPen(color)
+        for axis in self.plotItem.axes.values():
+            axis['item'].setPen(color)
 
     axisColor = Property(QColor, getAxisColor, setAxisColor)
 
@@ -731,8 +867,13 @@ class BasePlot(PlotWidget, PyDMPrimitiveWidget):
         self.clearAxes()
         for d in new_list:
             self.addAxis(plot_data_item=None, name=d.get('name'), orientation=d.get('orientation'),
-                         min_range=d.get('minRange'), max_range=d.get('maxRange'),
-                         enable_auto_range=d.get('autoRange'))
+                         label=d.get('label'), min_range=d.get('minRange'), max_range=d.get('maxRange'),
+                         enable_auto_range=d.get('autoRange'), log_mode=d.get('logMode'))
+        if 'bottom' in self.plotItem.axes:
+            # Ensure the added y axes get the color that was set
+            self.setAxisColor(self.getAxis('bottom')._pen.color())
+        if self.getShowYGrid() or self.getShowXGrid():
+            self.plotItem.updateGrid()
 
     yAxes = Property("QStringList", getYAxes, setYAxes, designable=False)
 
@@ -743,10 +884,10 @@ class BasePlot(PlotWidget, PyDMPrimitiveWidget):
         """
         Provide whether the right y-axis is being shown.
 
-        Returns : bool
+        Returns
         -------
-        True if the graph shows the right y-axis. False if not.
-
+        bool
+            True if the graph shows the right y-axis. False if not.
         """
         return self._show_right_axis
 
@@ -820,7 +961,8 @@ class BasePlot(PlotWidget, PyDMPrimitiveWidget):
             if len(self._y_labels) > 0:
                 # Hardcoded for now as we only have one axis
                 label = self._y_labels[0]
-            self.setLabel("left", text=label)
+            if 'left' in self.plotItem.axes:
+                self.setLabel("left", text=label)
 
     def resetYLabels(self):
         warnings.warn("Y Labels should now be set on the AxisItem itself. See: AxisItem.setLabel() "
@@ -834,8 +976,9 @@ class BasePlot(PlotWidget, PyDMPrimitiveWidget):
         """
         Check if the legend is being shown.
 
-        Returns : bool
+        Returns
         -------
+        bool
             True if the legend is displayed on the graph; False if not.
         """
         return self._show_legend
